/**
 * Kopf- und Fuß-Zeilen in HTML-Tabellen fixieren
 *
 * Autor:    Wilfried Schollenberger
 *           WS Unternehmensberatung und Controlling-Systeme GmbH
 * Kontakt:  wisch@ws-unternehmensberatung.de
 * Lizenz:   LGPL 3.0
 *
 * Stand:    13.07.2019 / 6.6.2022
 *
 * Das Script definiert eine Klasse TableHeader, die Kopf-
 * und Fuß-Zeilen am oberen bzw. unteren Fensterrand anzeigt,
 * Wenn die Tabelle über das Anzeige-Fenster hinausragt.
 *
 * Kopf und Fuß-Zeilen werden an der Auszeichnung mit
 * <thead> und <tfoot> erkannt.
 *
 * Die Funktion wird aktiviert, indem eine Instanz dieser Klasse
 * mit der ID der Tabelle instanziiert wird. Beispiel:
 *
 * <table id="Tabelle"> ... 
 *
 * <sript..>
 *	  new TableHeader("Tabelle"); 	
 *
 * Weiterhin können feste Zeilen-Labels definiert werden, die beim
 * Scrollen nach links stehen bleiben.
 * Diese Zellen (TH oder TD) müssen die Klasse "fts" erhalten.
 * (Kann in der Konstanten fixeSpaltenKlasse geändert werden.) 
 *
 * Beispiel:
 * <th class="fts">text</th>
 *
 * Der Klassenname wird in der Konstanten fixeSpaltenKlasse definiert 
 * und kann dort geändert werden.
 *
 * Um zu verhindern, dass diese Elemente die Druckausgabe stören, muss
 * eine CSS-Klasse noprint wie folgt angelegt werden:
 *
 *  @media print {
 *    .noprint {
 *       display: none;				
 *    }
 *   }
 *
 * Wenn die Tabelleninhalte von einer JavaScript-Routine geändert werden, muss diese
 * Routine auch für die Aktualisierung der fixierten Zellen sorgen. Dazu muss eine 
 * Referenz auf den TableHeader erzeugt werden:
 *
 * <script..>
 *	var th = new TableHeader("Tabelle"); 
 *
 * Wenn Inhalte von fixierten Zellen geändert wurde, muss
 *      th.refresh();
 * aufgerufen werden. 
 * Wenn nur "normale" Zellen geändert wurden, reicht der Aufruf von
 *      th.resize();
 *
 *
 * Prinzip:
 * Von den Elementen THEAD und TFOOT und den Label-Zellen werden mit cloneNode()
 * Kopien erzeugt und bei Bedarf an der richtigen Stelle angezeigt;
 *
 * Zur einfacheren Handhabung werden hier am Ende des Scripts
 * für die IDs "Tabelle", "Tabelle1" und "Tabelle2" entsprechende
 * TableHeader-Instanzen angelegt.
 *
 * Wenn die Tabellen nicht vorhanden sind oder keine thead/tfoot-Auszeichnungen
 * enthalten, passiert nichts.
 *
 * Neu 16.02.2022:
 * Bei fixierten Zeilen-Labels erhalten die Zellen Referenzen auf die Kopie bzw.
 * das Original.
 * Die kopierte Zelle erhält das Attribut THoriginal, welches auf die ursprüngliche
 * Zelle verweist.
 * Die originale Zelle aus dem HTML-Source erhält das Attribut THcopy, 
 * welches auf die Kopie zur fixierten Anzeige verweist.
 *
 * Neu 30.04.2022:
 * Auch die Zeile erhält ein THcopy für den TableFolder.
 * Die Tabelle erhält eine Referenz auf die zuständige Instanz des Tabellen Headers:
 * WSTableHeader, damit der TableFolder die resize()-Funktion aufrufen kann. 
 *
 * Korrekrur 9.5.2022:
 * Neuer Algorithmus bei der Positionierung der Zellen in fixierten Spalten
 * in Funktion scroll().
 * Jetzt können auch einzelne Spalten (oder Zellen) beim Fixieren ausgelassen
 * werden. 
 * Die Zellen werden jetzt über kZelle.getBoundingClientRect().left nebeneinander
 * gesetzt.
 *
 * Bug/Problem:
 *
 * 1.) Wenn die Tabellen-Breite über CSS in % des ParentElements gesetzt wird
 *     geht das in den Kopien schief.
 *     Als schnelle Umgehung wird deshalb die Klasse volleBreite aus dem
 *     Attribut className der Kopien entfernt.
 *     EINE LÖSUNG STEHT AUS !!!
 *
 * 2.) Es kann vorkommen, dass die Überschriften zuerst falsch dimensioniert sind,
 *     wenn die TableHeader über das Event DOMContentLoaded erzeugt werden.
 *     Testweise wurde deshalb auf window.addEventListener('load', function() {
 *     umgestellt.
 *     IM TEST
 *  
 * 3.) Bei den fixierten Zeilen-Label muss explizit eine Hintergrund-Farbe gesetzt
 *     werden. Deshalb kann dort diese Farbe nicht über dynamisches CSS, z.B.
 *     TH:hover{} gesetzt werden. 
 *     Wenn im CSS ein hover bei tr oder th und td definiert ist, muss zusätzlich 
 *     eine .hover-Klasse definiert werden. Beispiel:
 *     tr:hover, tr.hover{ background-color: #aaa;}
 *
 *     Dann kann die Hover-Funktion im TableHeader aktiviert werden. Beispiel:
 *        var th = new TableHeader('MeineTabelle');
 *        th.aktiviereHover(th.ZEILEN);
 *
 *     alternativ: th.ZELLEN und th.BEIDES
 *   
 *     Bei den Standard-Namen Tabelle und Tabelle1 bis Tabelle4 funktioniert das mit 
 *        var th = document.getElementById('Tabelle').WSTableHeader;
 *        th.aktiviereHover(th.ZEILEN);
 * 
 *     Anmerkung: Beim Instanziieren des Objekts funktioniert die Funktion noch nicht.
 *                Das Registrieren der Event-Listener funktioniert offenbar erst, 
 *                wenn die Instanziierung des Objekts abgeschlossen ist. (???)
 *	
 * 4.) Attribute der TR-, THEAD-, TBODY- und TFOOT- Tags werden bei fixierten Spalten
 *     nicht übernommen, weil dort nur die Zellen selbst angezeigt werden.
 *      
 * 5.) Wenn TableHeader und TableFolder für eine Tabelle verwendet werden, 
 *     muss der TableHeader vor dem TableFolder instanziiert werden.
 * 
 * 6.) Styles für TR funktioneren nicht, well nur Zellen und keine Zeilen angezeigt
 *     werden. 
 *     Lösung ist in Arbeit, aner aktuell niedrige Priorität.    
 *     
 */

function TableHeader(tabellenID) {

	// Tabelle, TableHeader, -footer und die Clones
	var tabelle = null;
	var tabTHead = null;
	var tabTFoot = null;
	// erster und letzter Tabellenabschnitt (thead, tbody, tfoot)
	// zur Steuerung der festen Head- und Foot-Abschnitte
	var tabTFirst = null;
	var tabTLast = null;
	
	var tabUeberschrift = null;
	var tabTHeadUeberschrift = null;
	
	var tabFusszeile = null;
	var tabTFootFusszeile = null;

	var fixeSpalten = false;

	var fixeBodyTabelle = null;
	var fixeHeadTabelle = null;
	var fixeFootTabelle = null;
	var hoverMode = 0;

	// Klassenname und Arrays für fixierte Zellen	
	const fixeSpaltenKlasse = "fts";
	var zeilen = Array();
	var kopierteZeilen = Array();

	// Die folgenden Werte können über Table- oder Style-Attribute
	// spezifiziert sein
	var border = 0;
	var cellSpacing = 2;
	var padding = 0;  
	var tBorder = 0; // Border-Breite für kopierte Kopf- und Fußzeilen



	// Tabelle suchen
	tabelle = document.getElementById(tabellenID);
	if (tabelle == null) return;  // KEINE TABELLE GEFUNDEN

	console.log('Initialisiere TableHeader für '+tabellenID);
	
	if (tabelle.attributes.hasOwnProperty("border")) border = parseFloat(tabelle.border);
	if (tabelle.style.border > "") border = parseFloat(tabelle.style.border);

	if (tabelle.attributes.hasOwnProperty("cellSpacing")) cellSpacing = parseFloat(tabelle.cellSpacing);
	if (tabelle.style.borderSpacing > "") cellSpacing = parseFloat(tabelle.style.borderSpacing);

	if (tabelle.style.padding > "") padding = parseFloat(tabelle.style.padding);


	// TBody-Abschnitte suchen suchen
	var tbodies = tabelle.getElementsByTagName("tbody");	 
	tabTFirst = tbodies [0];
	tabTLast = tbodies [tbodies.length - 1];
	

	// Überschriften suchen
	tabTHead =  tabelle.getElementsByTagName("thead")[0];
	if (tabTHead != null) {
		tabTFirst = tabTHead;
		// Kopie für feste Kopfzeile erstellen und in parentNode eintragen,
		// damit eventuelle Styles übernommen werden 
		tabTHeadUeberschrift = tabTHead.cloneNode(true);
		tabUeberschrift = tabelle.cloneNode(false);
		if (border > 1) {
			setBorder1(tabUeberschrift);	
		}
		// Bug-Umgehung, weil mit der Klasse volleBreite die Tabellenbreite gesetzt wird
	    tabUeberschrift.className = tabUeberschrift.className.replace("volleBreite","");

		tabUeberschrift.id = tabellenID+"_Header";
		tabUeberschrift.appendChild(tabTHeadUeberschrift);
		tabUeberschrift.hidden = true;
		tabUeberschrift.style.position = "fixed";
		tabUeberschrift.style.tableLayout = "fixed";
		tabUeberschrift.style.boxSizing = "border-box";
		if (tabUeberschrift.style.padding > "") tabUeberschrift.style.paddingBottom = "0px";
		tabUeberschrift.style.backgroundColor = getSolidBackground(tabelle);
		tabelle.parentNode.appendChild(tabUeberschrift);
		
		// Die feste Kopfzeile darf nicht gedruckt werden
		tabUeberschrift.classList.add('noprint');				
	}
	
	// Fußzeilen suchen
	tabTFoot =  tabelle.getElementsByTagName("tfoot")[0];
	if (tabTFoot != null) {
		tabTLast = tabTFoot;
		// Kopie für feste Fußzeile erstellen und in parentNode eintragen,
		// damit eventuelle Styles übernommen werden  
		tabTFootFusszeile = tabTFoot.cloneNode(true);
		tabFusszeile = tabelle.cloneNode(false);
		if (border > 1) {
			setBorder1(tabFusszeile);
		}
		// Bug-Umgehung, weil mit der Klasse volleBreite die Tabellenbreite gesetzt wird
	    tabFusszeile.className = tabFusszeile.className.replace("volleBreite","");
		
		tabFusszeile.id = tabellenID+"_Footer";
		tabFusszeile.appendChild(tabTFootFusszeile);
		tabFusszeile.hidden = true;		
		tabFusszeile.style.position = "fixed";
		tabFusszeile.style.tableLayout = "fixed";
		tabFusszeile.style.boxSizing = "border-box";
 		if (tabFusszeile.style.padding > "") 
  				tabFusszeile.style.paddingTop = "0px";
		tabFusszeile.style.backgroundColor = getSolidBackground(tabelle);		
		tabelle.parentNode.appendChild(tabFusszeile);		
		
		// Die feste Fußzeile darf nicht gedruckt werden
		tabFusszeile.classList.add('noprint');			
	}

	// Fixierte Spalten suchen
	fixeBodyTabelle = makeEmptyCopy(tabelle);
	fixeSpalten = fixeSpaltenSuchen(tabelle,fixeBodyTabelle);	
	if (fixeSpalten) {
		var inserted = false;
		
		if (tabUeberschrift != null) {
			tabelle.parentElement.insertBefore(fixeBodyTabelle,tabUeberschrift);
			inserted = true;
			fixeHeadTabelle = makeEmptyCopy(tabUeberschrift);
			fixeSpaltenSuchen(tabUeberschrift,fixeHeadTabelle);
			tabelle.parentElement.appendChild(fixeHeadTabelle);
		}
		if (tabFusszeile != null) {
			if (!inserted) tabelle.parentElement.insertBefore(fixeBodyTabelle,tabFusszeile);
			inserted = true; 		
			fixeFootTabelle = makeEmptyCopy(tabFusszeile); 
			fixeSpaltenSuchen(tabFusszeile,fixeFootTabelle);
			tabelle.parentElement.appendChild(fixeFootTabelle);
		}
		if (!inserted) tabelle.parentElement.appendChild(fixeBodyTabelle);
	}
	// Erweiterung für Tabellen-Folder
	tabelle.WSTableHeader = this;
	// Spalten-Breiten in den Kopien initialisieren
	resize();
	
	// Eventlistener registrieren
	window.addEventListener('scroll', scroll);
	window.addEventListener('resize', resize);
	console.log('TableHeader für '+tabellenID+' ist fertig.');

/**
 * Aus Performancegründen müssen die Zellen der fixen Spalten in leeren
 * Tabellen stehen, damit hidden nur einmal geändert wird.
 * Sonst stockt es.
 * Hier wird eine Kopie der Tabelle muster erzeugt.
 * Die Style-Attribute werden so gesetzt, dass nichts angezeigt wird.
 */
 
	function makeEmptyCopy(muster) {
		var tab = muster.cloneNode(false);
		tab.id += "_fix";
		tab.classList.add('noprint');
		tab.hidden = true;
		tab.style.position = "fixed";
		tab.style.left="-100px";
		tab.style.top="-100px";
		tab.style.width="10px";
		tab.style.height="10px";
		tab.style.tableLayout = "fixed";
		return tab;
	}  


/**
 * Event-Listener
 */

/**
 * Nach Scrollen und Resize müssen der Tabellenkopf und -footer aktualisiert werden
 * Dazu wird diese Funktion als Eventlistener registriert und von der Funktion resize() aufgerufen.
 */
 
 
 function scroll() {
 	// Tabellen-Position und Header-Größe bestimmen.
  		var boundingRect = tabelle.getBoundingClientRect();
 		var windowHeight = document.querySelector('html').clientHeight;  
		var headerHeight = 0;
		
	// Kopfzeilen behandeln
		if (tabUeberschrift != null) {
			// Die Kopien sind noch nicht gerendert. Deshalb Höhe des THEAD-Elements
			var boundingRectH = tabTHead.getBoundingClientRect();
			headerHeight = boundingRectH.height;  		
			if (boundingRectH.top  < 0 
				&& (tabTLast.getBoundingClientRect().bottom > boundingRectH.height) ) {
				// Feste Kopfzeile ausrichten und anzeigen					
				tabUeberschrift.style.left = (boundingRect.left	+ border - tBorder) + "px";
				tabUeberschrift.style.top = "0px";		
				tabUeberschrift.hidden=false;
//				tabUeberschrift.style.width = tabelle.offsetWidth+"px";	
			}else{
				// Feste Kopfzeile ausblenden
				tabUeberschrift.hidden = true;
			}
		}

	// Fußzeilen behandeln
		if (tabFusszeile != null) {		
			var boundingRectF = tabTFoot.getBoundingClientRect();	
			var footerHeight = boundingRectF.height;
			if (tabTFirst.getBoundingClientRect().top < (windowHeight - footerHeight - headerHeight 
				- 1 * cellSpacing)//	- border ) 
					&& boundingRectF.bottom > windowHeight) {
				// Feste Fußzeile ausrichten und anzeigen					
				tabFusszeile.style.left = (boundingRect.left + border - tBorder) + "px";
				tabFusszeile.style.top = (windowHeight - tabFusszeile.offsetHeight + tBorder)+"px";
				tabFusszeile.hidden=false;
			}else{
				// Feste Fußzeile ausblenden
				tabFusszeile.hidden = true;
			}
		}
		
	// Fixe Spalten behandeln	
		if (fixeSpalten) {	
			var shift = 0;				// .left-Postion der nächsten Zelle in Pixel
			var shiftPos = Array();		// .left-Position der ersten Zelle in der nächsten Zeile in shiftPos[shiftCountPos]
			var shiftCount = Array();	// Zeilen-Zähler für die Wirkung von Spalten mit rowSpan > 1 
			var shiftCountPos = -1;		// Zeiger auf aktuellen Wert von shiftPos
			var tpos = tabelle.getBoundingClientRect();
			if (tpos.left < -1 * (border + cellSpacing + padding)) {
				for (var i = 0; i < zeilen.length; i++) {
					var zeile = zeilen[i];
					if (! zeile.hidden){
						var kopie = kopierteZeilen[i];
						shift = 0;
						if (shiftCountPos >= 0) shift = shiftPos[shiftCountPos];	
						for (var z = 0; z < kopie.length; z++) {						
							var kZelle = kopie[z]; 						
							if (kZelle != null){
								var oZelle = kZelle.THoriginal;
								var zpos = oZelle.getBoundingClientRect();		
							    kZelle.style.width = (zpos.width + cellSpacing)+"px";
							    kZelle.style.height= (zpos.height)+"px";
								
							    kZelle.style.left = shift+'px';
								// shift für aktuelle Zeile erhöhen
								shift += zpos.width + cellSpacing ;
								// Bei rowSpan > 1 muss die Breite ddieser Zeile in der nächsten Zeile
								// berücksichtigt werden.		
								if (kZelle.rowSpan > 1){						
									shiftCountPos ++;
									shiftCount[shiftCountPos] = kZelle.rowSpan;
									shiftPos[shiftCountPos] =  zpos.width + cellSpacing ;
									if (shiftCountPos > 0) shiftPos[shiftCountPos] += shiftPos[shiftCountPos-1];
								}
						   		kZelle.style.top = (zpos.top)+"px"; 
								kZelle.style.backgroundColor = getSolidBackground(oZelle);
						   
								if (zpos.top <= windowHeight && zpos.bottom >=0){
	// Zellen-Ausrichtung über padding simulieren.
									var vAlign = window.getComputedStyle(oZelle).verticalAlign;
									var ktHeight = oZelle.firstChild.offsetHeight; // Höhe des Zelleninhalts
							
									if (vAlign == "middle"){
										kZelle.style.paddingTop = ((oZelle.clientHeight - ktHeight)/2) + "px"; 
									}else if (vAlign == "bottom"){
										var paddingBottom = 
											parseFloat(window.getComputedStyle(oZelle).paddingBottom)
										kZelle.style.paddingTop = (oZelle.clientHeight
													 - ktHeight - paddingBottom) + "px";
									}				   
								}
						 	}else console.log('Fehler bei Zeile '+1+' Zelle '+z+' kZelle ist NULL');  
						} // Schleife über kopierte Zellen einer Zeile			
						// Zeilen-Zähler für die Wirkung von Spalten mit rowSpan > 1 
						// um 1 zurücksetzen.
						// Aktuellen Zeiger shiftCountPos ermitteln 																			
						for (var ip = shiftCountPos; ip >= 0; ip --) {						
							shiftCount[ip] --	
							if (shiftCount[ip] <= 0) shiftCountPos --;
						}
					} // If Not zeile.hidden 
				} // For-Schleife über zeilen / kopierteZeilen			

				// Fixe Zellen anzeigen				
				fixeBodyTabelle.hidden=false;
				if (tabUeberschrift != null)					
					fixeHeadTabelle.hidden = tabUeberschrift.hidden;
				if (tabFusszeile != null)					
					fixeFootTabelle.hidden = tabFusszeile.hidden;
			}else{
			// Fixe Zellen ausblenden				
				fixeBodyTabelle.hidden = true;
				if (tabUeberschrift != null)					
					fixeHeadTabelle.hidden = true;
				if (tabFusszeile != null)					
					fixeFootTabelle.hidden = true;
			}
		}
	}


/**
 * Nur zur Initialisierung und nach Resize müssen die
 * Größen der Header- und Footer-Elemente ermittelt werden.
 * Dazu wird dieser Eventlistener registriert und beim
 * Initialisieren aufgerufen.
 */
 
	function resize () {
		// Header-Tabelle dimensionieren
		if (tabUeberschrift != null) {
		    tabUeberschrift.style.width=null;
		    tabUeberschrift.removeAttribute("width");
  			copyWidth(tabTHead,tabTHeadUeberschrift);
		}
	
		// Footer-Tabelle dimensionieren
		if (tabFusszeile != null) {
			tabFusszeile.style.width=null;
			tabFusszeile.removeAttribute("width");
			copyWidth(tabTFoot,tabTFootFusszeile);
		}
		scroll();
	}


/**
 * Fixierte Spalten suchen
 */
	
	function fixeSpaltenSuchen (element,fixeTabelle) {
		var sfix = false;
		var i = 0;
		if (element.tagName == "TR") {  // ZEILE gefunden
			i = zeilen.length;
			zeilen[i] = element;
			var kopie = Array();
			kopierteZeilen[i] = kopie;
			var zeile = document.createElement("tr");
			var spalten = element.children;    
			i = 0;
			for (var ic=0; ic < spalten.length; ic++) {
				var sp = spalten[ic];
				if (sp.classList.contains(fixeSpaltenKlasse)) {
					sfix = true;
					var spKopie = sp.cloneNode(true);
					spKopie.style.position = "fixed";    
					spKopie.classList.add('noprint');
					spKopie.style.boxSizing = "border-box";
					if (tabelle.border > "0" && spKopie.style.border == "" ) 
							spKopie.style.border = "1px solid";
					sp.innerHTML = "<div>"+sp.innerHTML+"</div>";
					if (hasSolidBackground(spKopie)) spKopie.THsolidbg = true;
					else{
						spKopie.style.backgroundColor = getSolidBackground(sp);
						spKopie.THsolidbg = false;
					} 
					spKopie.THoriginal = sp;
					sp.THcopy = spKopie;
					
					if (cellSpacing > 0){
							var paddingRight = 0;
							if (tabelle.cellPadding > "") paddingRight = parseFloat(tabelle.cellPadding);
							if (spKopie.style.paddingRight > "") paddingRight = parseFloat(spKopie.style.paddingRight);
							spKopie.style.paddingRight = (cellSpacing + paddingRight) + "px";
					}
//					fixeZelleAusrichten(sp,spKopie);    						
					kopie[i] = spKopie;
					zeile.appendChild(spKopie)
					i++;
				}				
			}
			element.THcopy = zeile;										
			fixeTabelle.appendChild(zeile);			
		}else if(element.hasChildNodes()) {
			var children = element.children;
			for (i=0; i < children.length; i++) {
				if (fixeSpaltenSuchen(children[i],fixeTabelle)) sfix = true;
			}	
		}	
		return sfix;
	}


/**
 * Hilfsfunktionen
 */


/*
 * Hintergrundfarbe eines HTML-Nodes ermitteln.
 * Entweder ist sie explizit gesetzt, oder sie muss über den
 * Hierarchiebaum parentElement ermittelt werden
 */

	function getSolidBackground(element) {
		while (element != null) {
			var bg = window.getComputedStyle(element).backgroundColor; 
			if (bg != "transparent"	&& bg.substr(0,4) != "rgba"){
				return bg;
			}
			element = element.parentElement;
		}
		return "white";
	}

/*
 * Prüfen, ob die zu kopierende Zelle einen nicht transparenten
 * hintergrund hat
 */
	function hasSolidBackground(element) {
		var bg = window.getComputedStyle(element).backgroundColor; 
		if (bg != "transparent"	&& bg.substr(0,4) != "rgba"){
				return false;
			}
		return true;
	}


/*
 * Setzt die Breite (width) der Elemente in cnode auf den Wert von offsetWidth aus pnode.
 * Immer wirksam ist das nur, wenn style.minWidth und style.maxWidth gleichzeitig gesetzt werden.
 * Die Funktion wird rekursiv aufgerufen
 * Leider funktioniert es nicht mit dem Tabellen-Element (table), sondern erst mit dem THEAD-Element
 * als Start-Node. 
 */
	function copyWidth(pnode, cnode) {
		// nur "echte" Elemente berückichtigen	
		if (cnode.nodeType != 1) return;
	
		if (cnode.tagName == "TH" || cnode.tagName == "TD") {
				cnode.style.boxSizing = "border-box";
				cnode.style.width = pnode.offsetWidth+"px" ;
				cnode.style.maxWidth = pnode.offsetWidth+"px" ;
				cnode.style.minWidth = pnode.offsetWidth+"px" ;
//				if (tabelle.border > "0" && cnode.style.border == "" ) cnode.style.border = "1px solid black";
				return;
		} 

		for (var i=0; i < cnode.childNodes.length; i++) {
			if (cnode.childNodes.item(i).childNodes.length > 0) {	
						copyWidth(pnode.childNodes.item(i),cnode.childNodes.item(i));						
			}				
		}
	}
	
	
/*
 * Das Border-Attribut wird für die Kopien auf 1 zurückgesetzt.
 */
  function setBorder1(tab){
		if (tab.style.border > "") 
		  tab.style.border = "1px "+tab.style.border.substr(tab.style.border.indexOf(" "));
		else  
		if (tab.attributes.hasOwnProperty("border")) tab.border = "1";
		else tab.style.border = "1px solid black";
		tBorder = 1;
  }
	 
/*
 * öffentliche Funktionsaufrufe zum Aktualisieren
 * nach Änderungen der Tabelleninhalte
 */
 
 // Neu berechnen der Zellengrößen	 
  this.resize = function () { resize();}
  
  
  
 // Aktualisieren der Inhalte von fixierten Zellen   
  this.refresh = function(){
		if (tabUeberschrift != null)	refresh(tabTHead,tabTHeadUeberschrift);  	
		if (tabFusszeile != null)	refresh(tabTFoot,tabTFootFusszeile);
		if (fixeSpalten) {
			for (var i=0; i < zeilen.length;i++) {
					var zeile = zeilen[i]; 
					var kopie = kopierteZeilen[i];
					var zellen = zeile.children;												
					for (var z = 0; z < kopie.length; z++) {	
						kopie[z].innerHTML = zellen.item(z).innerHTML;
						var ih = zellen.item(z).innerHTML;
						var il = ih.length;
						if (il < 12) zellen.item(z).innerHTML = "<div>"+il+"</div>";
						else 
						if (ih.substr(0,5) != "<div>" || ih.substr(il-6,6) != "</div>")   
								zellen.item(z).innerHTML = "<div>"+il+"</div>";
					}
			}
		}
		resize();  	
  	}
   // rekursiver Funktionsaufruf
     function refresh(pnode,cnode){
		// nur "echte" Elemente berückichtigen	
		if (cnode.nodeType != 1) return;
	
		if (cnode.tagName == "TH" || cnode.tagName == "TD") {
				cnode.innerHTML = pnode.innerHTML;
				return;
		} 

		for (var i=0; i < cnode.childNodes.length; i++) {
			if (cnode.childNodes.item(i).childNodes.length > 0) {	
						refresh(pnode.childNodes.item(i),cnode.childNodes.item(i));						
			}				
		}
  
  }

/*
 * Hover-Funktionen für fixierte Spalten
 * Konstanten in Deutsch und Englisch
 * wir sind fehlertolerant :-)
 */
 this.ZELLEN = 1;
 this.CELLS  = 1;
 this.ZEILEN = 2;
 this.ROWS   = 2;
 this.BEIDES = 3;
 this.BEIDE  = 3;
 this.BOTH   = 3;

 var hoverMode = 0;
/*
 * Funktionen für tr:hover
 */
    function setBackgroundF(tr){  
	  if (! (hoverMode & 2)) return;
      var nl = tr.childNodes;
      for (var i = 0; i < nl.length; i++){
        var cn = nl[i];
        if (cn.THcopy != null) {
            cn.THcopy.style.backgroundColor = getSolidBackground(cn);
        }
      }
    }

    function copyBackground(event){
      setBackgroundF(event.target);
    }

    function resetBackground(event){  
      setBackgroundF(event.target);
    }

   function setHover(event){
     var o = event.target;
     if (o.THoriginal != null){
		if (hoverMode & 2){
	        var row = o.THoriginal.parentNode;
	        row.classList.add('hover');
	        setBackgroundF(row);			
		}
		if (hoverMode & 1){
        	o.style.backgroundColor = null;
	        o.classList.add('hover');
		}
     }
    }

    function resetHover(event){
     var o = event.target;
     if (o.THoriginal != null){
		if (hoverMode & 1){
	        o.classList.remove('hover');
		}
		if (hoverMode & 2){
	        var row = o.THoriginal.parentNode;
	        row.classList.remove('hover');
	        setBackgroundF(row);
		}else
        o.style.backgroundColor = getSolidBackground(o.THoriginal);
     }
    }
  

/*
 * EventListener registrieren
 */
function activateHover(mode){
	  hoverMode = mode;
      var zellen = tabelle.getElementsByClassName('fts');
      for (var i = 0; i < zellen.length; i++){
        var z = zellen[i];
        if (z.THcopy != null) {
          // nur für Kopien für fixierte Zeilen-Label
          z.THcopy.addEventListener('mouseenter',setHover,false);
          z.THcopy.addEventListener('mouseleave',resetHover,false);
        }
      }
      // Nur Zeilen der Original-Tabelle
      var zeilen = tabelle.getElementsByTagName('tr');
      for (var i = 0; i < zeilen.length; i++){
          var z = zeilen[i];
          z.addEventListener('mouseenter',copyBackground,false);
          z.addEventListener('mouseleave',resetBackground,false);
      }
} 

  this.activateHover = function(mode){activateHover(mode);}
  this.aktiviereHover = function(mode){activateHover(mode);}
  this.hoverAktivieren = function(mode){activateHover(mode);}

  
// zum Debuggen  
  this.getTabelle = function(){
  	return tabelle;
  }
  this.getTabTFoot = function(){
  	return tabTFoot;
  }
  this.getTabTHead = function(){
  	return tabTHead;
  }

/**
	Verschieben von Zellen für den TableFolder
	Beim Einklappen müssen die Label-Zellen aus der ersten Zeile der Gruppe
	in die letzte Zeile der Gruppe verschoben werden.
	beim Ausklappen wäre es genau umgekehrt.
	
	kopierteZeilen ist ein Array mit Arrays mit den fixierten Zellen.
	Deshalb reicht es, die Zellen zu kopieren und die kopierten
	Zellen wieder zu löschen.
	Wäre kopierteZeilen ist ein Array von TR-Elementen, würden die
	Zellen automatisch verschoben, weil jede Zelle nur einen Parent
	(TR-Element) haben kann. 
 */ 
  this.TFkopiere = function(count,von, nach){
	var zeileVon = 0;
	var zeileNach = 0;
	let fixeTabelle = von.parentElement.children;
	
	for (let z=0; z < fixeTabelle.length; z++){
		if (fixeTabelle.item(z) == von) zeileVon = z;
		if (fixeTabelle.item(z) == nach) zeileNach = z;		
	}
//	console.log('Kopiere '+count+' Zellen von '+zeileVon+' nach '+zeileNach);
	var zVon = kopierteZeilen[zeileVon];
	var zNach = kopierteZeilen[zeileNach];
	for (let c = count -1; c >= 0;c--){
		zNach.unshift(zVon[c]);
	}
	for (let c = count -1; c>0;c--){
//		zVon.shift();
	}
}

  this.TFloesche = function(count,zeile){
		let tabellenZeilen = zeile.parentElement.children;
		
		for (let z=0; z < tabellenZeilen.length; z++){
			if (tabellenZeilen.item(z) == zeile) {
//				console.log('Lösche '+count+' Zellen aus Zeile '+z);
				for (let c = count -1; c >= 0;c--){
					kopierteZeilen[z].shift();
				}
				return;		
			}
		}
	}
}	// Ende der Klassen-Definition


/**
 * TableHeader für 3 Namen aktivieren
 */ 
// document.addEventListener('DOMContentLoaded', function() {
window.addEventListener('load', function() {
 	new TableHeader("Tabelle"); 	
 	new TableHeader("Tabelle1"); 	 
 	new TableHeader("Tabelle2"); 	
 	new TableHeader("Tabelle3"); 	
 	new TableHeader("Tabelle4"); 	
 });
